﻿using UnityEngine;
using System.Collections;

public class Enemy : MonoBehaviour {
	private float duration = 3f;
	private Vector3[] path;
	
	private int[,] curMap;
	
	private Vector3 startPos;
	private Vector3 endPos;
	private float startTime;
	private int curWaypoint;
	bool move;
	
	private GameObject[] eyes;
	private int eyeDelay = 0;
	
	private bool followPlayer = true;
	private int pathRecalculationTimer = 0;
	private int pathRecalculationPause = 120;
	
	private GameObject player;
	
	
	void Start () {
		Levels levelScript = GameObject.Find("Level").GetComponent("Levels") as Levels;
		curMap = levelScript.getWalkables();
		
		eyes = GameObject.FindGameObjectsWithTag("EnemyEye");
		player = GameObject.FindGameObjectWithTag("Player");
		
		//To prevent all enemies calculating at the same frame 
		pathRecalculationTimer = Random.Range(0, pathRecalculationPause);
		Debug.Log("Enemy random: "+ pathRecalculationTimer);
	}
	
	void Update () {
		if (followPlayer && !Registry.stageClear && !Registry.stageFailed && !Registry.gameOver) {
			pathRecalculationTimer++;
			if (pathRecalculationTimer >= pathRecalculationPause) {
				Pathfinding pathFinding = new Pathfinding();
				path = pathFinding.findPath(transform.position, player.transform.position, curMap);
				if (path != null) {
					startMovingByPath();
					pathRecalculationTimer = 0;
				}
			}
		}
		if (Registry.stageClear || Registry.stageFailed || Registry.gameOver) {
			move = false;	
		}
		
		
		if (move) {
			var i = (Time.time - startTime) * duration;
			transform.position = Vector3.Lerp(startPos, endPos, i);
			transform.LookAt(endPos);
			
			if (i >= 1) {
				startTime = Time.time;
				curWaypoint++;
				if (curWaypoint >= path.Length) {
					move = false;
				} else {
					startPos = endPos;
					endPos = path[curWaypoint];
				}
			}
		}
		
		//Blink eyes
		eyeDelay++;
		if (eyeDelay == 10) {
			if (Random.Range(0,20) == 0) {
				foreach (GameObject eye in eyes) {
					eye.renderer.material.color = Color.black;
				}
			} else if (eyeDelay >= 10) {
				foreach (GameObject eye in eyes) {
					eye.renderer.material.color = Color.yellow;
				}
			}
			eyeDelay = 0;
		}
	}
	
	
	void startMovingByPath() {
		if (path.Length > 0) {
			startPos = transform.position;
			startTime = Time.time;
			curWaypoint = 0;
			endPos = path[curWaypoint];
			move = true;
		}
	}
}

