﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;


public class Pathfinding  {

	private List<Node> openNodes;
	private List<Node> closedNodes;
	
	
	public Vector3[] findPath(Vector3 fromPos, Vector3 toPos, int[,]map) {
		
		fromPos = new Vector3(Mathf.Round(fromPos.x),Mathf.Round(fromPos.y),Mathf.Round(fromPos.z));
		toPos = new Vector3(Mathf.Round(toPos.x),Mathf.Round(toPos.y),Mathf.Round(toPos.z));
		
		if (fromPos != toPos) {
						
			int mapWidth = map.GetLength(1);
			int mapHeight = map.GetLength(0);
			
			openNodes = new List<Node>();
			closedNodes = new List<Node>();
			
			bool noFinishFound = true;
			
			Node newNode = new Node();
			newNode.pos = fromPos;
			openNodes.Add(newNode);

			int max = 1000;
			while (noFinishFound && openNodes.Count > 0 && max > 0) {
				int curNodeIndex = findLowestScore();
				Node curNode = openNodes[curNodeIndex];
				closedNodes.Add (openNodes[curNodeIndex]);
				openNodes.RemoveAt(curNodeIndex);
				if (curNode.dist == 0) {
					noFinishFound = false;	
				} else {
					
					//check nodes around current
					int curX = (int)Mathf.Round(curNode.pos.x);
					int curZ = (int)Mathf.Round(curNode.pos.z);
					//right
					if (curX + 1 < mapWidth && 
						curX + 1 >= 0 && 
						map[curZ, curX+1] == 0) {
						
						newNode = createNode(curNode.pos, toPos, curNode.step+10, 1f, 0);
						addToLists(newNode);
					}
					
					//lower right
					if (curX + 1 < mapWidth && 
						curX + 1 >= 0 && 
						curZ - 1 < mapWidth && 
						curZ - 1 >= 0 &&
						map[curZ, curX+1] == 0 &&
						map[curZ-1, curX] == 0 &&
						map[curZ-1, curX+1] == 0) {
						
						newNode = createNode(curNode.pos, toPos, curNode.step+14, 1f, -1f);
						addToLists(newNode);
					}
					
					//down
					if (curZ - 1 < mapHeight && 
						curZ - 1 >= 0 && 
						map[curZ-1, curX] == 0) {
						newNode = createNode(curNode.pos, toPos, curNode.step+10, 0, -1f);
						addToLists(newNode);
					}
					
					//lower left
					if (curX - 1 < mapWidth && 
						curX - 1 >= 0 && 
						curZ - 1 < mapWidth && 
						curZ - 1 >= 0 &&
						map[curZ, curX-1] == 0 &&
						map[curZ-1, curX] == 0 &&
						map[curZ-1, curX-1] == 0) {
						
						newNode = createNode(curNode.pos, toPos, curNode.step+14, -1f, -1f);
						addToLists(newNode);
					}
					
					//left
					if (curX - 1 < mapWidth &&
						curX - 1 >= 0 &&
						map[curZ, curX-1] == 0) {
						newNode = createNode(curNode.pos, toPos, curNode.step+10, -1f, 0);
						addToLists(newNode);
					}
					
					//top left
					if (curX - 1 < mapWidth && 
						curX - 1 >= 0 && 
						curZ + 1 < mapWidth && 
						curZ + 1 >= 0 &&
						map[curZ, curX-1] == 0 &&
						map[curZ+1, curX] == 0 &&
						map[curZ+1, curX-1] == 0) {
						
						newNode = createNode(curNode.pos, toPos, curNode.step+14, -1f, 1f);
						addToLists(newNode);
					}
					
					//up
					if (curZ + 1 < mapHeight && 
						curZ + 1 >= 0 && 
						map[curZ+1, curX] == 0) {
						newNode = createNode(curNode.pos, toPos, curNode.step+10, 0, +1f);
						addToLists(newNode);
					}
					
					//top right
					if (curX + 1 < mapWidth && 
						curX + 1 >= 0 && 
						curZ + 1 < mapWidth && 
						curZ + 1 >= 0 &&
						map[curZ, curX+1] == 0 &&
						map[curZ+1, curX] == 0 &&
						map[curZ+1, curX+1] == 0) {
						
						newNode = createNode(curNode.pos, toPos, curNode.step+14, 1f, 1f);
						addToLists(newNode);
					}
				}
				max--;			
			}
	
			if (!noFinishFound) {
				List<Vector3> path = new List<Vector3>();
				int curNodeIndex = findNodeWithPosition(closedNodes, toPos);
				
				path.Add(closedNodes[curNodeIndex].pos);
				for (int i = 0; i < closedNodes.Count; i++) {
					//Get the next node, which is the owner of the current
					curNodeIndex = findNodeWithPosition(closedNodes, closedNodes[curNodeIndex].own);
					if (curNodeIndex == 0) {
						break;	
					}
					path.Add(closedNodes[curNodeIndex].pos);
				}
				path.Reverse();
				return path.ToArray();
				
			} else {
				return null;
			}
		}
		return null;
	}
	
	int getDistance (Vector3 fromPos, Vector3 toPos) {
		int x = (int)(Mathf.Abs(toPos.x - fromPos.x) + 0.5f);
		int y = (int)(Mathf.Abs(toPos.z - fromPos.z) + 0.5f);
		return x+y;
	}
				
	Node createNode (Vector3 owner, Vector3 toPos, int step, float xOffset, float zOffset) {
		Vector3 newPos = new Vector3(owner.x + xOffset, owner.y, owner.z + zOffset);
		int dist = getDistance(newPos, toPos) * 10;
		return new Node(newPos, owner, step, dist, step+dist);
	}
	
	int findLowestScore() {
		int lowest = openNodes[0].total;
		int id = 0;
		for (int i = 1; i < openNodes.Count; i++) {
			if (openNodes[i].total < lowest) {
				lowest = openNodes[i].total;
				id = i;
			}
		}
		return id;
	}
		
	void addToLists (Node node) {
		int openIndex = findObjectInList(openNodes, node);
		if (openIndex >= 0) {
			if (openNodes[openIndex].total > node.total)
				openNodes.Add(node);
		} else if (findObjectInList(closedNodes, node) >= 0) {
			//do nothing if already closed
		} else {
			openNodes.Add(node);
		}		
	}
	
	int findObjectInList(List<Node> list, Node node) {
		for (int i = 0; i < list.Count; i++) {
			if (list[i].pos.Equals(node.pos)) {
				return i;
			}
		}
		return -1;
	}
	
	int findNodeWithPosition(List<Node> list, Vector3 vector) {
		for (int i = 0; i < list.Count; i++) {
			if (list[i].pos.Equals(vector)) {
				return i;	
			}
		}
		return -1;
	}
}
