﻿using UnityEngine;
using System.Collections;

public class Player : MonoBehaviour {
	
	public GameObject pathPlupp;
	private float duration = 3f;
	private GameObject pathPluppContainer;
	private Vector3[] path;
	
	private int[,] curMap;
	
	private Vector3 startPos;
	private Vector3 endPos;
	private float startTime;
	private int curWaypoint;
	bool move;
	
	private GameObject cam;
	private GameObject flash;
	private int flashdelay = 0;
	private GameObject[] eyes;
	private int eyeDelay = 0;
	
	
	void Start () {
		Levels levelScript = GameObject.Find("Level").GetComponent("Levels") as Levels;
		curMap = levelScript.getWalkables();
		
		cam = GameObject.FindWithTag("MainCamera");
		
		flash = GameObject.FindWithTag("Flash");
		eyes = GameObject.FindGameObjectsWithTag("PlayerEye");
		
		flash.renderer.enabled = false;
		setCameraPos();
	}
	
	void Update () {
		if (!Registry.stageClear && !Registry.stageFailed && !Registry.gameOver) {
			
			if (Input.GetMouseButtonDown(0)) {
				clearPath();
				Ray ray = Camera.main.ScreenPointToRay( Input.mousePosition );
			    RaycastHit hit;
			 
			    if( Physics.Raycast( ray, out hit, 100 ) )
			    {
					if (hit.transform.tag.Equals("Floor")) {
						Pathfinding pathFinding = new Pathfinding();
						path = pathFinding.findPath(transform.position, hit.point, curMap);
						if (path != null) {
							drawPath(path);
							startMovingByPath();
						}
					}
			    }	
			}
			
			if (move) {
				var i = (Time.time - startTime) * duration;
				transform.position = Vector3.Lerp(startPos, endPos, i);
				transform.LookAt(endPos);
				setCameraPos();
				
				
				if (i >= 1) {
					startTime = Time.time;
					curWaypoint++;
					if (curWaypoint >= path.Length) {
						move = false;
						clearPath();
					} else {
						startPos = endPos;
						endPos = path[curWaypoint];
					}
				}
			}
			
			//Blink eyes
			eyeDelay++;
			if (eyeDelay == 10) {
				if (Random.Range(0,20) == 0) {
					foreach (GameObject eye in eyes) {
						eye.renderer.material.color = Color.white;
					}
				} else if (eyeDelay >= 10) {
					foreach (GameObject eye in eyes) {
						eye.renderer.material.color = Color.black;
					}
				}
				eyeDelay = 0;
			}
	
			//Flash
			if (flash.renderer.enabled) {
				if (flashdelay == 0) {
					flash.renderer.enabled = false;	
				} else {
					flashdelay--;
				}
			}
		}
		
		if (Registry.stageClear || Registry.stageFailed || Registry.gameOver) {
			clearPath();
		}
	}
	
	
	void OnCollisionEnter (Collision collision) {
		Debug.Log("Collision Player with: " + collision.collider.name);
		if (collision.collider.tag.Equals("Enemy")) {
			audio.Play();
			flash.renderer.enabled = true;
			flashdelay = 5;
			Registry.health -= 25;
			Registry.score -= 50;
		}
		
		if (collision.collider.tag.Equals("Exit")) {
			Registry.stageClear = true;	
		}
	}
	
	void OnTriggerEnter (Collider collider) {
		Debug.Log ("Trigger :D");	
	}
	
	
	
	
	void setCameraPos() {
		//Set camera position
		float camX = transform.position.x + -10f;
		float camY = transform.position.y + 50f;
		float camZ = transform.position.z + -10f;
		cam.transform.position = new Vector3(camX, camY, camZ);	
	}
	
	
	void startMovingByPath() {
		if (path.Length > 0) {
			startPos = transform.position;
			startTime = Time.time;
			curWaypoint = 0;
			endPos = path[curWaypoint];
			move = true;
		}
	}
	
	void drawPath(Vector3[] pathArr) {
		pathPluppContainer = Instantiate(new GameObject("PathPlupps")) as GameObject;
		foreach (Vector3 vector in pathArr) {
			GameObject curPlupp = Instantiate(pathPlupp, vector, new Quaternion()) as GameObject;
			curPlupp.transform.parent = pathPluppContainer.transform;
		}
	}
	
	void clearPath() {
		DestroyImmediate(pathPluppContainer);
		pathPluppContainer = null;
	}
	
}

